(*
Problem 04 - Text Processing

const
	kMaxHandleSize = 1000000;
const
	kActionMove = 1;
	kActionInsert = 2;
	kActionDelete = 3;
	kActionSearch = 4;
	kActionSearchAndReplace = 5;
const
	kFlagCaseSenstitiveBit = 0;
	kFlagGlobalBit = 1;
	kFlagBackwardsBit = 2;
type
  ActionRecord = record
    action: UInt32;
    amount: SInt32;
    flags: UInt32;
    search: Str255;
    replace: Str255;
  end;
  ActionRecordArray = array[0..0] of ActionRecord;
  ActionRecordArrayPtr = ^ActionRecordArray;

procedure TextProcess( data: Handle; action_count: UInt32; actions: ActionRecordArrayPtr );

Given an unlocked handle to a block of text, you need to apply a sequence of actions to the data.  Internally, you must keep a current position pointer, 
which starts at zero (ie, before the first character in the handle).  The internal position pointer is constrained between 0 and GetHandleSize(data) at all times, 
and if it becomes less than zero (respectively greater than GetHandleSize), then you must reset it to zero (respectively GetHandleSize).  
The handle size will never exceed 1 Meg, and you will have plenty of memory to play with.  Characters from chr(127)-chr(255) will never appear in the handle 
or any strings.  In response to each of the actions you need to apply the following action:

kActionMove - move the internal position forward by the amount field (which may be negative).  (Example, kActionMove with amount set to kMaxHandleSize will set the internal pointer to be GetHandleSize( data )).

kActionInsert - Insert the replace string at the current location.  (Example, if the internal pointer is GetHandleSize( data ), then the replace string will be appended to the handle.

kActionDelete - Delete the number of characters specified by the amount field.  The internal pointer is never moved.  If the amount field is greater than the number of characters after the internal pointer, then fewer characters are deleted such that the handle is truncated at the internal pointer.

kActionSearch - search forward (backwards if the kFlagBackwardsBit is set in the flags field) for the search field (case sensitively if the kFlagCaseSenstitiveBit is set in the flags field).  The position pointer should be set to point before the first character of the string if it is found, or at GetHandleSize (zero if backwards) if not found.  If there is a match at the initial position, it is not counted (ie, kActionMove -kMaxHandleSize, kSearch 'hello', kSearch 'hello' finds the second occurance of 'hello').

kActionSearchAndReplace - as for search, but when a match is found, replace it with the replace string.  If the kFlagGlobalBit is set in the flags field, continue search until a match is not found but never search any characters of the replace string. (that is, if you replace 'aaa' with 'ABA' (case insensitively) in 'aaaaaaaa' you will get 'ABAABAaa', not 'ABABABAa')

[wow this needs some rewriting]
[hmm, perhaps a text input would be better:
MOVE<space><decimal number>
INSERT<space><text>
DELETE<space><decimal number>
SEARCH<space>BACKWARDS<space>CASESENSITIVE<space><text>
REPLACE<space>BACKWARDS<space>CASESENSITIVE<space>GLOBAL<space><text><seperator?><replace>
]
*)

unit Solution;

interface

// Do not modify the interface

	uses
		Types, Files;
		
	const
		kMaxHandleSize = 1000000;
	const
		kActionMove = 1;
		kActionInsert = 2;
		kActionDelete = 3;
		kActionSearch = 4;
		kActionSearchAndReplace = 5;
	const
		kFlagCaseSenstitiveBit = 0;
		kFlagGlobalBit = 1;
		kFlagBackwardsBit = 2;
	type
	  ActionRecord = record
	    action: UInt32;
	    amount: SInt32;
	    flags: UInt32;
	    search: Str255;
	    replace: Str255;
	  end;
	  ActionRecordArray = array[0..0] of ActionRecord;
	  ActionRecordArrayPtr = ^ActionRecordArray;

	procedure TextProcess( data: Handle; action_count: UInt32; actions: ActionRecordArrayPtr );

implementation

// Fill in your solution and then submit this folder

// Team Name: Example Solution
// 25 minutes

	uses
		TextUtils;

	function Pin( a, b, c: longint ): longint;
	begin
		if b < a then begin
			b := a;
		end else if b > c then begin
			b := c;
		end;
		Pin := b;
	end;

	function MatchAt( data: Handle; pos: longint; casesensitive: boolean; search: string ): boolean;
		var
			t: str255;
	begin
		MatchAt := false;
		if (0 <= pos) & (pos + length(search) <= GetHandleSize( data )) then begin
			t[0] := search[0];
			BlockMoveData( Ptr(longint(data^)+pos), @t[1], length(search) );
			if not casesensitive then begin
				UpperString( t, false );
				UpperString( search, false );
			end;
			MatchAt := t = search;
		end;
	end;

	procedure TextProcess( data: Handle; action_count: UInt32; actions: ActionRecordArrayPtr );
		var
			curpos: longint;
			action: ActionRecord;
			casesensitive, global, backwards: boolean;
		
		function DoSearch: boolean;
		begin
			DoSearch := false;
			if backwards then begin
				Dec(curpos);
				while curpos >= 0 do begin
					if MatchAt( data, curpos, casesensitive, action.search ) then begin
						DoSearch := true;
						leave;
					end;
					Dec(curpos);
				end;
			end else begin
				Inc(curpos);
				while curpos < GetHandleSize( data ) do begin
					if MatchAt( data, curpos, casesensitive, action.search ) then begin
						DoSearch := true;
						leave;
					end;
					Inc(curpos);
				end;
			end;
			curpos := Pin( 0, curpos + action.amount, GetHandleSize(data) );
		end;
		
		var
			junkl, i: longint;
			found: boolean;
	begin
		curpos := 0;
		for i := 1 to action_count do begin
			action := actions^[i-1];
			casesensitive := btst(action.flags,kFlagCaseSenstitiveBit);
			global := btst(action.flags,kFlagGlobalBit);
			backwards := btst(action.flags,kFlagBackwardsBit);
			case action.action of
				kActionMove: begin
					curpos := Pin( 0, curpos + action.amount, GetHandleSize(data) );
				end;
				kActionInsert: begin
					junkl := Munger( data, curpos, nil, 0, @action.replace[1], length(action.replace) );
				end;
				kActionDelete: begin
					if action.amount > 0 then begin
						if curpos + action.amount >= GetHandleSize( data ) then begin
							SetHandleSize( data, curpos );
						end else begin
							junkl := Munger( data, curpos, nil, action.amount, @junkl, 0 );
						end;
					end;
				end;
				kActionSearch: begin
					found := DoSearch;
				end;
				kActionSearchAndReplace: begin
					while true do begin
						found := DoSearch;
						if found then begin
							junkl := Munger( data, curpos, nil, length(action.search), @action.replace[1], length(action.replace) );
							if global then begin
								if backwards then begin
									curpos := curpos - length(action.search) + 1;
								end else begin
									curpos := curpos + length(action.search) - 1;
								end;
							end else begin
								leave;
							end;
						end else begin
							leave;
						end;
					end;
					curpos := Pin( 0, curpos + action.amount, GetHandleSize(data) );
				end;
				otherwise begin
					DebugStr( 'bad' );
				end;
			end;
		end;
	end;
	
end.
